@@ -251,7 +251,109 @@ class TxTOrderDataset(Dataset):
             else:
                 raise NotImplementedError()
 
+class KRTxTOrderDataset(TxTOrderDataset):
+    """ OrderDataset class for KR dataset """
+    def __init__(self, ctx_cols, seq_col, vl_col, history_col, label_col,
+                 is_train=True, from_s3=True, local_data_dir=None, local_lookup_dirs=None,
+                 s3_bucket=None, s3_prefix_data=None, s3_prefix_lookups=None,
+                 parts_num=9999, data_file_format="json", lookup_file_format="json"):
+        self.is_train = is_train
+        self.history_col = history_col
+        super().__init__(ctx_cols, seq_col, vl_col, label_col,
+                 from_s3, local_data_dir, local_lookup_dirs,
+                 s3_bucket, s3_prefix_data, s3_prefix_lookups,
+                 parts_num, data_file_format, lookup_file_format)
+        if self.history_col:
+            self.history_np = np.concatenate(self.history_np, axis=0)
 
+    def _valid_len(self, row):
+        seq = row[self.seq_col]
+        seq = [p for p in seq if p != 0]
+        vl = len(seq)
+        return vl
+
+    def _append_np_from_df(self, df):
+        # generate validate length col
+        df[self.vl_col] = df.apply(self._valid_len, axis=1)
+        # delete the rows where the validate length = 0
+        df = df[df[self.vl_col] != 0]
+        self.ctx_np.append(np.array(df[self.ctx_cols]))
+        self.seq_np.append(np.array(df[self.seq_col].to_list()))
+        self.vl_np.append(np.array(df[self.vl_col]))
+        self.label_np.append(np.array(df[self.label_col].astype(float)))
+        if self.history_col:
+            self.history_np.append(np.array(df[self.history_col]))
+
+    def _load_data_from_s3(self):
+        # devide into train and test (little data)
+        keys = []
+        s3_client = boto3.client('s3')
+        resp = s3_client.list_objects_v2(Bucket=self.s3_bucket, Prefix=self.s3_prefix_data)
+        for obj in resp['Contents']:
+            # for train or test
+            if self.is_train:
+                if obj['Key'].endswith(self.data_file_format) and "part-00148" not in obj['Key']:
+                    keys.append(obj['Key'])
+            else:
+                if obj['Key'].endswith(self.data_file_format) and "part-00148" in obj['Key']:
+                    keys.append(obj['Key'])
+        for i, key in enumerate(keys):
+            obj = s3_client.get_object(Bucket=self.s3_bucket, Key=key)
+            df = self._read_df(obj['Body'], file_format=self.data_file_format)
+            self._append_np_from_df(df)
+            if i > self.parts_num:
+                break
+
+    def _load_data_from_local(self):
+        # divide into train and test (little data)
+        if self.is_train:
+            f_paths = [
+                os.path.join(self.local_data_dir, f_name)
+                for f_name in os.listdir(self.local_data_dir)
+                if f_name.endswith(self.data_file_format) and "part-00148" not in f_name
+            ]
+        else:
+            f_paths = [
+                os.path.join(self.local_data_dir, f_name)
+                for f_name in os.listdir(self.local_data_dir)
+                if f_name.endswith(self.data_file_format) and "part-00148" in f_name
+            ]
+        for i, f_path in enumerate(f_paths):
+            df = self._read_df(f_path, self.data_file_format)
+            self._append_np_from_df(df)
+            if i > self.parts_num:
+                break
+
+    def get_feature_num(self, col):
+        # TODO: implement via lookup dict
+        # data_col = self.data[col]
+        # if data_col.dtype == "int64":
+        #     return data_col.astype(int).max() + 1
+        if col == self.ctx_cols[0]:
+            return self.ctx_np[:, 0].astype(int).max() + 1
+        elif col == self.ctx_cols[1]:
+            return self.ctx_np[:, 1].astype(int).max() + 1
+        elif col == self.seq_col:
+            return int(self.seq_np.reshape(-1).max()) + 1
+        else:
+            raise NotImplementedError()
+
+    def export_lookups(self, local_dir, file_format):
+        if not os.path.exists(local_dir):
+            os.makedirs(local_dir)
+        for lookup_name, lookup_df in self.lookups.items():
+            lookup_name = lookup_name.split("_")[0]
+            output_path = os.path.join(local_dir, f"{lookup_name}.{file_format}")
+            # add col names and change col dtype
+            lookup_df.columns = ["value", "index"]
+            lookup_df = lookup_df.astype({"value": str, "index": int})
+            if file_format == "json":
+                lookup_df.to_json(output_path)
+            elif file_format == "csv":
+                lookup_df.to_csv(output_path, index=False)
+            else:
+                raise NotImplementedError()
+                
 class PLTxT(LightningModule):
     def __init__(self, loss, optimizer, learning_rate, top_k, txt_kwargs):
         super().__init__()
